#include "configeditor.h"

#include <QFile>
#include <QFileDialog>
#include <QFormLayout>
#include <QGroupBox>
#include <QMessageBox>
#include <QTextCodec>
#include <QTextStream>
#include <QToolButton>
#include <QtDebug>
#include "rpfile.h"

XkorConfigEditor::XkorConfigEditor(QWidget * parent) : QWidget(parent)
{
	fileIsModified = false;
	
	configName = new QLineEdit;
	startListDirectory = new XkorFileLineEdit(tr("Select start list directory"), ".");
	
	// tree widget
	rpbonus = new XkorRPBonusWidget;
	
	// form layout
	QFormLayout * form = new QFormLayout;
	form->addRow(tr("Name:"), configName);
	form->addRow(tr("Start list directory:"), startListDirectory);
	
	// tree widget layout
	QGroupBox * rpgroup = new QGroupBox(tr("RP bonuses"));
	QHBoxLayout * rplayout = new QHBoxLayout;
	rplayout->addWidget(rpbonus);
	rpgroup->setLayout(rplayout);
	
	// layout
	QVBoxLayout * layout = new QVBoxLayout(this);
	layout->addLayout(form, 0);
	layout->addWidget(rpgroup, 1);
	
	connect(configName, SIGNAL(textChanged(QString)), this, SLOT(setModified()));
	connect(startListDirectory, SIGNAL(textChanged(QString)), this, SLOT(setModified()));
	connect(rpbonus, SIGNAL(listChanged()), this, SLOT(setModified()));
}

bool XkorConfigEditor::okayToLoad()
{
	if(fileIsModified)
	{
		int r = QMessageBox::warning(this, tr("xkoranate"), tr("The config file has been modified.\nDo you want to save your changes?"), QMessageBox::Yes | QMessageBox::Default, QMessageBox::No, QMessageBox::Cancel | QMessageBox::Escape);
		if(r == QMessageBox::Cancel)
			return false;
	}
	return true; // if the current file hasn’t been modified, or if the user clicked Yes or No
}

void XkorConfigEditor::openFile()
{
	if(okayToLoad())
	{
		QString filename = QFileDialog::getOpenFileName(this, tr("Open config file"), ".", tr("XML files (*.xml)"));
		
		try
		{
			XkorConfig conf(filename);
			configName->setText(conf.getName());
			startListDirectory->setText(conf.getStartListDirectory());
			
			std::map<QString, double> bonuses = conf.getAllBonuses();
			rpbonus->setBonuses(bonuses);
			
			currentFileName = filename;
			emit fileChanged(filename);
			setModified(false);
		}
		catch(XkorFileNotFoundException & e)
		{
			qDebug() << e.what();
			QMessageBox::critical(this, tr("xkoranate"), tr("Specified %1 (%2) not found.").arg(e.getFileType()).arg(e.getFileName()));
		}
	}
}

void XkorConfigEditor::saveFile(QString filename)
{	
	if(filename == QString()) // if no filename was specified
		filename = currentFileName;
	QFile f(filename);
	writeFile(&f);
	currentFileName = filename;
	emit fileChanged(filename);
	setModified(false);
}

void XkorConfigEditor::saveFileAs()
{
	QString filename = QFileDialog::getSaveFileName(this, tr("Save config file"), ".", tr("XML files (*.xml)"));
	if(filename != QString())
		saveFile(filename);
}

void XkorConfigEditor::setModified(bool isModified)
{
	fileIsModified = isModified;
	emit fileEdited(isModified);
}

void XkorConfigEditor::writeFile(QIODevice * dev)
{
	dev->open(QIODevice::WriteOnly);
	
	QTextStream ost(dev);
	ost.setCodec(QTextCodec::codecForName("UTF-8"));
	
	ost << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<rpbonus>\n";
	ost << "\t<name>" << configName->text() << "</name>\n";
	ost << "\t<startListDirectory>" << startListDirectory->text() << "</startListDirectory>\n";
	std::map<QString, double> bonuses = rpbonus->bonuses();
	for(std::map<QString, double>::iterator i = bonuses.begin(); i != bonuses.end(); ++i)
	{
		ost << "\t<nation>\n";
		ost << "\t\t<code>" << i->first << "</code>\n";
		ost << "\t\t<bonus>" << i->second << "</bonus>\n";
		ost << "\t</nation>\n";
	}
	ost << "</rpbonus>\n";
}
